
# Operation type                          
use constant BACKUP => 1;
use constant RESTORE => 2;
use constant LOCALSEARCH => 3;
use constant REMOTESEARCH => 4;
use constant DELETE => 5;
use constant ARCHIVE => 6;
use constant CLEANUP => 7;

# regex constants will not work.
my $backup = "Backup";
my $restore = "Restore";
my $archive = "Archive";
my $localSearch = "/Backup/.+searchList";
my $remoteSearch = "/Restore/.+searchList";
my $delete = "Delete";
my $cleanup = "+Cleanup";
my $operationType = 0;
my $lineFeed = "\n";
my $jobStatusCmd = "";
my $jobtype = "";
	
my $currentFilePath  = __FILE__;
require substr($currentFilePath, 0, rindex($currentFilePath, '/')) . '/fetchPath.pl';
		
my $jobName = `pwd`;
chomp($jobName);
getOperationType($jobName);

my $pidPath = "$jobName/pid.txt";
my $jobTermPath = "$jobName/utf8.txt";
my $curFile = `basename '$currentFilePath'`;

# check if pid.txt exist and if so then terminate the running operation
if(!-e $pidPath){
	traceLog("$jobName is not running", __FILE__, __LINE__);
	exit;
}

constructJobTerminateCommand();

if($jobStatusCmd eq ""){
	unlink($pidPath);	
	unlink($jobTermPath);
}
else {
	$psResult = `$jobStatusCmd`;
	@arr = split(/\n/, $psResult);
	chomp(@arr);
	$len = @arr;
	createUserCancel();
	unlink($pidPath);
	
	for($i = 0; $i < $len; $i++) {
		@tmpArr = split($separator, $arr[$i]);
		for($j = $pidIdx; $j <=$#tmpArr; $j++ ){
			if($tmpArr[$j] ne "" and ($tmpArr[$j] =~ /^\d+$/)) {
					`kill -9 $tmpArr[$j]`;
					last;
				}
		}
	}
	
	unlink($jobTermPath);
	traceLog("$jobName has been killed", __FILE__, __LINE__);
}

removeCutoffCronEntry();

#****************************************************************************************************
# Subroutine Name         : getOperationType.
# Objective               : Get operation type based on job path.
# Added By                : Avinash Kumar
#*****************************************************************************************************/
sub getOperationType()
{
	$jobPath = $_[0];

	if($jobPath =~ /$localSearch/){
		$operationType = LOCALSEARCH;
	}
	elsif($jobPath =~ /$remoteSearch/){
		$operationType = REMOTESEARCH;
	}
	elsif($jobPath =~ /$backup/){
		$operationType = BACKUP;
	}
	elsif($jobPath =~ /$restore/){
		$operationType = RESTORE;
	}
	elsif($jobPath =~ /$delete/){
		$operationType = DELETE;
	}
	elsif($jobPath =~ /$archive/){
		$operationType = ARCHIVE;		
	}
}

#****************************************************************************************************
# Subroutine Name         : createUserCancel.
# Objective               : Generate cancel.txt for backup and restore operations
# Added By                : Avinash Kumar
#*****************************************************************************************************/
sub createUserCancel()
{
	if(BACKUP != $operationType and RESTORE != $operationType){
		return;
	}
	
	if(open(CFH, ">> $userCancel")){
		if("" ne $ARGV[0]){
			print CFH "Operation could not be completed. Reason : Operation Cancelled due to Cut off.";
		}
		else {
			print CFH "Operation could not be completed, Reason: Operation Cancelled by User.";
		}
		close CFH;
		chmod $permissionVal, $userCancel; 
	}
	else {
		traceLog("Could not open file $userCancel, Reason:$!", __FILE__, __LINE__);
	}
}

#****************************************************************************************************
# Subroutine Name         : removeCutoffCronEntry.
# Objective               : remove cut off entry from crontab
# Added By                : Pooja H
#*****************************************************************************************************/
sub removeCutoffCronEntry()
{
	#In case of Job_termination for Start immediately option, cron entry for cut off will be removed
	if($ARGV[0] ne "" and $ARGV[0] == 0){
		my $temp_file = "$baseDir/$webDir/$SERVICE/$USER_PROFILE/$username/temp_operationsfile.txt";
		if(open(TEMP_FILE, "> $temp_file")){
			print TEMP_FILE "removeCrontabEntry\n"; 
			print TEMP_FILE '"'.$jobPath.'"';
			close TEMP_FILE;
			chmod $permissionVal, $temp_file;
			$operationsFile = "$baseDir/$webDir/$APPDIR/.scripts/operations.pl"; 
			exec("$perlPath $operationsFile");	
			traceLog("Could not exec operations.pl", __FILE__, __LINE__);
		}
		else {
			traceLog("Could not open file $temp_file, Reason:$!", __FILE__, __LINE__);
		}
	}
}
#****************************************************************************************************
# Subroutine Name         : removeCutoffCronEntry.
# Objective               : remove cut off entry from crontab
# Added By                : Pooja H
#*****************************************************************************************************/
sub constructJobTerminateCommand()
{
	if(BACKUP == $operationType or RESTORE == $operationType or REMOTESEARCH == $operationType or ARCHIVE == $operationType){
		$jobtype = "BACKUP_RESTORE_REMOTESEARCH_CLEANUP";
		$jobTermPath =~ s/'/'\\''/;
		$jobTermPath = "'".$jobTermPath."'";
		$jobStatusCmd = getJobTerminationCommand($operationType, $jobTermPath, $jobtype);
	}
	elsif(LOCALSEARCH == $operationType){
		$jobtype = "LOCALSEARCH";
		$jobName =~ s/'/'\\''/;
		$jobName = "'".$jobName."'";
		$jobStatusCmd = getJobTerminationCommand($operationType, $jobName, $jobtype);
	}
	elsif(DELETE == $operationType){
		$jobtype = "DELETE";
		$jobStatusCmd = getJobTerminationCommand($operationType, $jobName, $jobtype);
	}
}
